@@ -53,36 +53,43 @@ struct ctdb_vacuum_handle {
 
 /*  a list of records to possibly delete */
 struct vacuum_data {
-	uint32_t vacuum_limit;
 	uint32_t repack_limit;
 	struct ctdb_context *ctdb;
 	struct ctdb_db_context *ctdb_db;
 	struct tdb_context *dest_db;
 	trbt_tree_t *delete_list;
-	uint32_t delete_count;
 	struct ctdb_marshall_buffer **vacuum_fetch_list;
 	struct timeval start;
 	bool traverse_error;
 	bool vacuum;
-	uint32_t total;
-	uint32_t vacuumed;
-	uint32_t copied;
-	uint32_t fast_added_to_vacuum_fetch_list;
-	uint32_t fast_added_to_delete_list;
-	uint32_t fast_deleted;
-	uint32_t fast_skipped;
-	uint32_t fast_error;
-	uint32_t fast_total;
-	uint32_t full_added_to_vacuum_fetch_list;
-	uint32_t full_added_to_delete_list;
-	uint32_t full_skipped;
-	uint32_t full_error;
-	uint32_t full_total;
-	uint32_t delete_left;
-	uint32_t delete_remote_error;
-	uint32_t delete_local_error;
-	uint32_t delete_deleted;
-	uint32_t delete_skipped;
+	struct {
+		struct {
+			uint32_t added_to_vacuum_fetch_list;
+			uint32_t added_to_delete_list;
+			uint32_t deleted;
+			uint32_t skipped;
+			uint32_t error;
+			uint32_t total;
+		} delete_queue;
+		struct {
+			uint32_t scheduled;
+			uint32_t skipped;
+			uint32_t error;
+			uint32_t total;
+		} db_traverse;
+		struct {
+			uint32_t total;
+			uint32_t remote_error;
+			uint32_t local_error;
+			uint32_t deleted;
+			uint32_t skipped;
+			uint32_t left;
+		} delete_list;
+		struct {
+			uint32_t vacuumed;
+			uint32_t copied;
+		} repack;
+	} count;
 };
 
 /* this structure contains the information for one record to be deleted */
@@ -99,6 +106,10 @@ struct delete_records_list {
 	struct vacuum_data *vdata;
 };
 
+static int insert_record_into_delete_queue(struct ctdb_db_context *ctdb_db,
+					   const struct ctdb_ltdb_header *hdr,
+					   TDB_DATA key);
+
 /**
  * Store key and header in a tree, indexed by the key hash.
  */
@@ -147,7 +158,7 @@ static int add_record_to_delete_list(struct vacuum_data *vdata, TDB_DATA key,
 	hash = ctdb_hash(&key);
 
 	if (trbt_lookup32(vdata->delete_list, hash)) {
-		DEBUG(DEBUG_INFO, (__location__ " Hash collission when vacuuming, skipping this record.\n"));
+		DEBUG(DEBUG_INFO, (__location__ " Hash collision when vacuuming, skipping this record.\n"));
 		return 0;
 	}
 
@@ -158,7 +169,7 @@ static int add_record_to_delete_list(struct vacuum_data *vdata, TDB_DATA key,
 		return -1;
 	}
 
-	vdata->delete_count++;
+	vdata->count.delete_list.total++;
 
 	return 0;
 }
@@ -200,8 +211,6 @@ static int add_record_to_vacuum_fetch_list(struct vacuum_data *vdata,
 	memcpy(old_size+(uint8_t *)vfl, rec, rec->length);
 	talloc_free(rec);
 
-	vdata->total++;
-
 	return 0;
 }
 
@@ -209,23 +218,39 @@ static int add_record_to_vacuum_fetch_list(struct vacuum_data *vdata,
 static void ctdb_vacuum_event(struct event_context *ev, struct timed_event *te,
 			      struct timeval t, void *private_data);
 
+static int vacuum_record_parser(TDB_DATA key, TDB_DATA data, void *private_data)
+{
+	struct ctdb_ltdb_header *header =
+		(struct ctdb_ltdb_header *)private_data;
+
+	if (data.dsize != sizeof(struct ctdb_ltdb_header)) {
+		return -1;
+	}
+
+	*header = *(struct ctdb_ltdb_header *)data.dptr;
+
+	return 0;
+}
 
 /*
  * traverse function for gathering the records that can be deleted
  */
-static int vacuum_traverse(struct tdb_context *tdb, TDB_DATA key, TDB_DATA data, void *private)
+static int vacuum_traverse(struct tdb_context *tdb, TDB_DATA key, TDB_DATA data,
+			   void *private_data)
 {
-	struct vacuum_data *vdata = talloc_get_type(private, struct vacuum_data);
+	struct vacuum_data *vdata = talloc_get_type(private_data,
+						    struct vacuum_data);
 	struct ctdb_context *ctdb = vdata->ctdb;
+	struct ctdb_db_context *ctdb_db = vdata->ctdb_db;
 	uint32_t lmaster;
 	struct ctdb_ltdb_header *hdr;
 	int res = 0;
 
-	vdata->full_total++;
+	vdata->count.db_traverse.total++;
 
 	lmaster = ctdb_lmaster(ctdb, &key);
 	if (lmaster >= ctdb->num_nodes) {
-		vdata->full_error++;
+		vdata->count.db_traverse.error++;
 		DEBUG(DEBUG_CRIT, (__location__
 				   " lmaster[%u] >= ctdb->num_nodes[%u] for key"
 				   " with hash[%u]!\n",
@@ -237,42 +262,29 @@ static int vacuum_traverse(struct tdb_context *tdb, TDB_DATA key, TDB_DATA data,
 
 	if (data.dsize != sizeof(struct ctdb_ltdb_header)) {
 		/* it is not a deleted record */
-		vdata->full_skipped++;
+		vdata->count.db_traverse.skipped++;
 		return 0;
 	}
 
 	hdr = (struct ctdb_ltdb_header *)data.dptr;
 
 	if (hdr->dmaster != ctdb->pnn) {
-		vdata->full_skipped++;
+		vdata->count.db_traverse.skipped++;
 		return 0;
 	}
 
-	if (lmaster == ctdb->pnn) {
-		/*
-		 * We are both lmaster and dmaster, and the record is empty.
-		 * So we should be able to delete it.
-		 */
-		res = add_record_to_delete_list(vdata, key, hdr);
-		if (res != 0) {
-			vdata->full_error++;
-		} else {
-			vdata->full_added_to_delete_list++;
-		}
+	/*
+	 * Add the record to this process's delete_queue for processing
+	 * in the subsequent traverse in the fast vacuum run.
+	 */
+	res = insert_record_into_delete_queue(ctdb_db, hdr, key);
+	if (res != 0) {
+		vdata->count.db_traverse.error++;
 	} else {
-		/*
-		 * We are not lmaster.
-		 * Add the record to the blob ready to send to the nodes.
-		 */
-		res = add_record_to_vacuum_fetch_list(vdata, key);
-		if (res != 0) {
-			vdata->full_error++;
-		} else {
-			vdata->full_added_to_vacuum_fetch_list++;
-		}
+		vdata->count.db_traverse.scheduled++;
 	}
 
-	return res;
+	return 0;
 }
 
 /*
@@ -319,8 +331,7 @@ static int delete_marshall_traverse_first(void *param, void *data)
 	struct delete_records_list *recs = talloc_get_type(param, struct delete_records_list);
 	struct ctdb_db_context *ctdb_db = dd->ctdb_db;
 	struct ctdb_context *ctdb = ctdb_db->ctdb;
-	struct ctdb_ltdb_header *header;
-	TDB_DATA tdb_data, ctdb_data;
+	struct ctdb_ltdb_header header;
 	uint32_t lmaster;
 	uint32_t hash = ctdb_hash(&(dd->key));
 	int res;
@@ -331,7 +342,8 @@ static int delete_marshall_traverse_first(void *param, void *data)
 		      (__location__ " Error getting chainlock on record with "
 		       "key hash [0x%08x] on database db[%s].\n",
 		       hash, ctdb_db->db_name));
-		recs->vdata->delete_skipped++;
+		recs->vdata->count.delete_list.skipped++;
+		recs->vdata->count.delete_list.left--;
 		talloc_free(dd);
 		return 0;
 	}
@@ -341,26 +353,13 @@ static int delete_marshall_traverse_first(void *param, void *data)
 	 * changed and that we are still its lmaster and dmaster.
 	 */
 
-	tdb_data = tdb_fetch(ctdb_db->ltdb->tdb, dd->key);
-	if (tdb_data.dsize < sizeof(struct ctdb_ltdb_header)) {
-		DEBUG(DEBUG_INFO, (__location__ ": record with hash [0x%08x] "
-				   "on database db[%s] does not exist or is not"
-				   " a ctdb-record.  skipping.\n",
-				   hash, ctdb_db->db_name));
+	res = tdb_parse_record(ctdb_db->ltdb->tdb, dd->key,
+			       vacuum_record_parser, &header);
+	if (res != 0) {
 		goto skip;
 	}
 
-	if (tdb_data.dsize > sizeof(struct ctdb_ltdb_header)) {
-		DEBUG(DEBUG_INFO, (__location__ ": record with hash [0x%08x] "
-				   "on database db[%s] has been recycled. "
-				   "skipping.\n",
-				   hash, ctdb_db->db_name));
-		goto skip;
-	}
-
-	header = (struct ctdb_ltdb_header *)tdb_data.dptr;
-
-	if (header->flags & CTDB_REC_RO_FLAGS) {
+	if (header.flags & CTDB_REC_RO_FLAGS) {
 		DEBUG(DEBUG_INFO, (__location__ ": record with hash [0x%08x] "
 				   "on database db[%s] has read-only flags. "
 				   "skipping.\n",
@@ -368,7 +367,7 @@ static int delete_marshall_traverse_first(void *param, void *data)
 		goto skip;
 	}
 
-	if (header->dmaster != ctdb->pnn) {
+	if (header.dmaster != ctdb->pnn) {
 		DEBUG(DEBUG_INFO, (__location__ ": record with hash [0x%08x] "
 				   "on database db[%s] has been migrated away. "
 				   "skipping.\n",
@@ -376,7 +375,7 @@ static int delete_marshall_traverse_first(void *param, void *data)
 		goto skip;
 	}
 
-	if (header->rsn != dd->hdr.rsn) {
+	if (header.rsn != dd->hdr.rsn) {
 		DEBUG(DEBUG_INFO, (__location__ ": record with hash [0x%08x] "
 				   "on database db[%s] seems to have been "
 				   "migrated away and back again (with empty "
@@ -404,10 +403,7 @@ static int delete_marshall_traverse_first(void *param, void *data)
 	 * on the record's dmaster.
 	 */
 
-	ctdb_data.dptr = tdb_data.dptr + sizeof(struct ctdb_ltdb_header);
-	ctdb_data.dsize = tdb_data.dsize - sizeof(struct ctdb_ltdb_header);
-
-	res = ctdb_ltdb_store(ctdb_db, dd->key, header, ctdb_data);
+	res = ctdb_ltdb_store(ctdb_db, dd->key, &header, tdb_null);
 	if (res != 0) {
 		DEBUG(DEBUG_ERR, (__location__ ": Failed to store record with "
 				  "key hash [0x%08x] on database db[%s].\n",
@@ -422,15 +418,12 @@ static int delete_marshall_traverse_first(void *param, void *data)
 skip:
 	tdb_chainunlock(ctdb_db->ltdb->tdb, dd->key);
 
-	recs->vdata->delete_skipped++;
+	recs->vdata->count.delete_list.skipped++;
+	recs->vdata->count.delete_list.left--;
 	talloc_free(dd);
 	dd = NULL;
 
 done:
-	if (tdb_data.dptr != NULL) {
-		free(tdb_data.dptr);
-	}
-
 	if (dd == NULL) {
 		return 0;
 	}
@@ -466,12 +459,11 @@ static int delete_queue_traverse(void *param, void *data)
 	struct ctdb_db_context *ctdb_db = dd->ctdb_db;
 	struct ctdb_context *ctdb = ctdb_db->ctdb; /* or dd->ctdb ??? */
 	int res;
-	struct ctdb_ltdb_header *header;
-	TDB_DATA tdb_data;
+	struct ctdb_ltdb_header header;
 	uint32_t lmaster;
 	uint32_t hash = ctdb_hash(&(dd->key));
 
-	vdata->fast_total++;
+	vdata->count.delete_queue.total++;
 
 	res = tdb_chainlock(ctdb_db->ltdb->tdb, dd->key);
 	if (res != 0) {
@@ -479,29 +471,22 @@ static int delete_queue_traverse(void *param, void *data)
 		      (__location__ " Error getting chainlock on record with "
 		       "key hash [0x%08x] on database db[%s].\n",
 		       hash, ctdb_db->db_name));
-		vdata->fast_error++;
+		vdata->count.delete_queue.error++;
 		return 0;
 	}
 
-	tdb_data = tdb_fetch(ctdb_db->ltdb->tdb, dd->key);
-	if (tdb_data.dsize < sizeof(struct ctdb_ltdb_header)) {
-		/* Does not exist or not a ctdb record. Skip. */
+	res = tdb_parse_record(ctdb_db->ltdb->tdb, dd->key,
+			       vacuum_record_parser, &header);
+	if (res != 0) {
 		goto skipped;
 	}
 
-	if (tdb_data.dsize > sizeof(struct ctdb_ltdb_header)) {
-		/* The record has been recycled (filled with data). Skip. */
-		goto skipped;
-	}
-
-	header = (struct ctdb_ltdb_header *)tdb_data.dptr;
-
-	if (header->dmaster != ctdb->pnn) {
+	if (header.dmaster != ctdb->pnn) {
 		/* The record has been migrated off the node. Skip. */
 		goto skipped;
 	}
 
-	if (header->rsn != dd->hdr.rsn) {
+	if (header.rsn != dd->hdr.rsn) {
 		/*
 		 * The record has been migrated off the node and back again.
 		 * But not requeued for deletion. Skip it.
@@ -527,9 +512,9 @@ static int delete_queue_traverse(void *param, void *data)
 			DEBUG(DEBUG_ERR,
 			      (__location__ " Error adding record to list "
 			       "of records to send to lmaster.\n"));
-			vdata->fast_error++;
+			vdata->count.delete_queue.error++;
 		} else {
-			vdata->fast_added_to_vacuum_fetch_list++;
+			vdata->count.delete_queue.added_to_vacuum_fetch_list++;
 		}
 		goto done;
 	}
@@ -542,9 +527,9 @@ static int delete_queue_traverse(void *param, void *data)
 			DEBUG(DEBUG_ERR,
 			      (__location__ " Error adding record to list "
 			       "of records for deletion on lmaster.\n"));
-			vdata->fast_error++;
+			vdata->count.delete_queue.error++;
 		} else {
-			vdata->fast_added_to_delete_list++;
+			vdata->count.delete_queue.added_to_delete_list++;
 		}
 	} else {
 		res = tdb_delete(ctdb_db->ltdb->tdb, dd->key);
@@ -554,25 +539,23 @@ static int delete_queue_traverse(void *param, void *data)
 			      (__location__ " Error deleting record with key "
 			       "hash [0x%08x] from local data base db[%s].\n",
 			       hash, ctdb_db->db_name));
-			vdata->fast_error++;
-		} else {
-			DEBUG(DEBUG_DEBUG,
-			      (__location__ " Deleted record with key hash "
-			       "[0x%08x] from local data base db[%s].\n",
-			       hash, ctdb_db->db_name));
-			vdata->fast_deleted++;
+			vdata->count.delete_queue.error++;
+			goto done;
 		}
+
+		DEBUG(DEBUG_DEBUG,
+		      (__location__ " Deleted record with key hash "
+		       "[0x%08x] from local data base db[%s].\n",
+		       hash, ctdb_db->db_name));
+		vdata->count.delete_queue.deleted++;
 	}
 
 	goto done;
 
 skipped:
-	vdata->fast_skipped++;
+	vdata->count.delete_queue.skipped++;
 
 done:
-	if (tdb_data.dptr != NULL) {
-		free(tdb_data.dptr);
-	}
 	tdb_chainunlock(ctdb_db->ltdb->tdb, dd->key);
 
 	return 0;
@@ -591,10 +574,8 @@ static int delete_record_traverse(void *param, void *data)
 	struct ctdb_db_context *ctdb_db = dd->ctdb_db;
 	struct ctdb_context *ctdb = ctdb_db->ctdb;
 	int res;
-	struct ctdb_ltdb_header *header;
-	TDB_DATA tdb_data;
+	struct ctdb_ltdb_header header;
 	uint32_t lmaster;
-	bool deleted = false;
 	uint32_t hash = ctdb_hash(&(dd->key));
 
 	res = tdb_chainlock(ctdb_db->ltdb->tdb, dd->key);
@@ -603,7 +584,9 @@ static int delete_record_traverse(void *param, void *data)
 		      (__location__ " Error getting chainlock on record with "
 		       "key hash [0x%08x] on database db[%s].\n",
 		       hash, ctdb_db->db_name));
-		vdata->delete_local_error++;
+		vdata->count.delete_list.local_error++;
+		vdata->count.delete_list.left--;
+		talloc_free(dd);
 		return 0;
 	}
 
@@ -612,46 +595,29 @@ static int delete_record_traverse(void *param, void *data)
 	 * changed and that we are still its lmaster and dmaster.
 	 */
 
-	tdb_data = tdb_fetch(ctdb_db->ltdb->tdb, dd->key);
-	if (tdb_data.dsize < sizeof(struct ctdb_ltdb_header)) {
-		DEBUG(DEBUG_INFO, (__location__ ": record with hash [0x%08x] "
-				   "on database db[%s] does not exist or is not"
-				   " a ctdb-record.  skipping.\n",
-				   hash, ctdb_db->db_name));
-		vdata->delete_skipped++;
-		goto done;
+	res = tdb_parse_record(ctdb_db->ltdb->tdb, dd->key,
+			       vacuum_record_parser, &header);
+	if (res != 0) {
+		goto skip;
 	}
 
-	if (tdb_data.dsize > sizeof(struct ctdb_ltdb_header)) {
-		DEBUG(DEBUG_INFO, (__location__ ": record with hash [0x%08x] "
-				   "on database db[%s] has been recycled. "
-				   "skipping.\n",
-				   hash, ctdb_db->db_name));
-		vdata->delete_skipped++;
-		goto done;
-	}
-
-	header = (struct ctdb_ltdb_header *)tdb_data.dptr;
-
-	if (header->flags & CTDB_REC_RO_FLAGS) {
+	if (header.flags & CTDB_REC_RO_FLAGS) {
 		DEBUG(DEBUG_INFO, (__location__ ": record with hash [0x%08x] "
 				   "on database db[%s] has read-only flags. "
 				   "skipping.\n",
 				   hash, ctdb_db->db_name));
-		vdata->delete_skipped++;
-		goto done;
+		goto skip;
 	}
 
-	if (header->dmaster != ctdb->pnn) {
+	if (header.dmaster != ctdb->pnn) {
 		DEBUG(DEBUG_INFO, (__location__ ": record with hash [0x%08x] "
 				   "on database db[%s] has been migrated away. "
 				   "skipping.\n",
 				   hash, ctdb_db->db_name));
-		vdata->delete_skipped++;
-		goto done;
+		goto skip;
 	}
 
-	if (header->rsn != dd->hdr.rsn + 1) {
+	if (header.rsn != dd->hdr.rsn + 1) {
 		/*
 		 * The record has been migrated off the node and back again.
 		 * But not requeued for deletion. Skip it.
@@ -663,8 +629,7 @@ static int delete_record_traverse(void *param, void *data)
 				   "migrated away and back again (with empty "
 				   "data). skipping.\n",
 				   hash, ctdb_db->db_name));
-		vdata->delete_skipped++;
-		goto done;
+		goto skip;
 	}
 
 	lmaster = ctdb_lmaster(ctdb_db->ctdb, &dd->key);
@@ -674,8 +639,7 @@ static int delete_record_traverse(void *param, void *data)
 				   "delete list (key hash [0x%08x], db[%s]). "
 				   "Strange! skipping.\n",
 				   hash, ctdb_db->db_name));
-		vdata->delete_skipped++;
-		goto done;
+		goto skip;
 	}
 
 	res = tdb_delete(ctdb_db->ltdb->tdb, dd->key);
@@ -685,47 +649,64 @@ static int delete_record_traverse(void *param, void *data)
 		      (__location__ " Error deleting record with key hash "
 		       "[0x%08x] from local data base db[%s].\n",
 		       hash, ctdb_db->db_name));
-		vdata->delete_local_error++;
+		vdata->count.delete_list.local_error++;
 		goto done;
 	}
 
-	deleted = true;
-
 	DEBUG(DEBUG_DEBUG,
 	      (__location__ " Deleted record with key hash [0x%08x] from "
 	       "local data base db[%s].\n", hash, ctdb_db->db_name));
 
-done:
-	if (tdb_data.dptr != NULL) {
-		free(tdb_data.dptr);
-	}
+	vdata->count.delete_list.deleted++;
+	goto done;
 
+skip:
+	vdata->count.delete_list.skipped++;
+
+done:
 	tdb_chainunlock(ctdb_db->ltdb->tdb, dd->key);
 
-	if (deleted) {
-		/*
-		 * successfully deleted the record locally.
-		 * remove it from the list and update statistics.
-		 */
-		talloc_free(dd);
-		vdata->delete_deleted++;
-		vdata->delete_left--;
-	}
+	talloc_free(dd);
+	vdata->count.delete_list.left--;
 
 	return 0;
 }
 
 /**
- * Fast vacuuming run:
  * Traverse the delete_queue.
- * This fills the same lists as the database traverse.
+ * Records are either deleted directly or filled
+ * into the delete list or the vacuum fetch lists
+ * for further processing.
  */
-static void ctdb_vacuum_db_fast(struct ctdb_db_context *ctdb_db,
-				struct vacuum_data *vdata)
+static void ctdb_process_delete_queue(struct ctdb_db_context *ctdb_db,
+				      struct vacuum_data *vdata)
 {
-	trbt_traversearray32(ctdb_db->delete_queue, 1, delete_queue_traverse, vdata);
+	uint32_t sum;
+	int ret;
 
-	if (vdata->fast_total > 0) {
+	ret = trbt_traversearray32(ctdb_db->delete_queue, 1,
+				   delete_queue_traverse, vdata);
+
+	if (ret != 0) {
+		DEBUG(DEBUG_ERR, (__location__ " Error traversing "
+		      "the delete queue.\n"));
+	}
+
+	sum = vdata->count.delete_queue.deleted
+	    + vdata->count.delete_queue.skipped
+	    + vdata->count.delete_queue.error
+	    + vdata->count.delete_queue.added_to_delete_list
+	    + vdata->count.delete_queue.added_to_vacuum_fetch_list;
+
+	if (vdata->count.delete_queue.total != sum) {
+		DEBUG(DEBUG_ERR, (__location__ " Inconsistency in fast vacuum "
+		      "counts for db[%s]: total[%u] != sum[%u]\n",
+		      ctdb_db->db_name,
+		      (unsigned)vdata->count.delete_queue.total,
+		      (unsigned)sum));
+	}
+
+	if (vdata->count.delete_queue.total > 0) {
 		DEBUG(DEBUG_INFO,
 		      (__location__
 		       " fast vacuuming delete_queue traverse statistics: "
@@ -737,43 +718,36 @@ static void ctdb_vacuum_db_fast(struct ctdb_db_context *ctdb_db,
 		       "adl[%u] "
 		       "avf[%u]\n",
 		       ctdb_db->db_name,
-		       (unsigned)vdata->fast_total,
-		       (unsigned)vdata->fast_deleted,
-		       (unsigned)vdata->fast_skipped,
-		       (unsigned)vdata->fast_error,
-		       (unsigned)vdata->fast_added_to_delete_list,
-		       (unsigned)vdata->fast_added_to_vacuum_fetch_list));
+		       (unsigned)vdata->count.delete_queue.total,
+		       (unsigned)vdata->count.delete_queue.deleted,
+		       (unsigned)vdata->count.delete_queue.skipped,
+		       (unsigned)vdata->count.delete_queue.error,
+		       (unsigned)vdata->count.delete_queue.added_to_delete_list,
+		       (unsigned)vdata->count.delete_queue.added_to_vacuum_fetch_list));
 	}
 
 	return;
 }
 
 /**
- * Full vacuum run:
  * read-only traverse of the database, looking for records that
  * might be able to be vacuumed.
  *
  * This is not done each time but only every tunable
  * VacuumFastPathCount times.
  */
-static int ctdb_vacuum_db_full(struct ctdb_db_context *ctdb_db,
-			       struct vacuum_data *vdata,
-			       bool full_vacuum_run)
+static void ctdb_vacuum_traverse_db(struct ctdb_db_context *ctdb_db,
+				    struct vacuum_data *vdata)
 {
 	int ret;
 
-	if (!full_vacuum_run) {
-		return 0;
-	}
-
 	ret = tdb_traverse_read(ctdb_db->ltdb->tdb, vacuum_traverse, vdata);
 	if (ret == -1 || vdata->traverse_error) {
 		DEBUG(DEBUG_ERR, (__location__ " Traverse error in vacuuming "
 				  "'%s'\n", ctdb_db->db_name));
-		return -1;
 	}
 
-	if (vdata->full_total > 0) {
+	if (vdata->count.db_traverse.total > 0) {
 		DEBUG(DEBUG_INFO,
 		      (__location__
 		       " full vacuuming db traverse statistics: "
@@ -781,17 +755,15 @@ static int ctdb_vacuum_db_full(struct ctdb_db_context *ctdb_db,
 		       "total[%u] "
 		       "skp[%u] "
 		       "err[%u] "
-		       "adl[%u] "
-		       "avf[%u]\n",
+		       "sched[%u]\n",
 		       ctdb_db->db_name,
-		       (unsigned)vdata->full_total,
-		       (unsigned)vdata->full_skipped,
-		       (unsigned)vdata->full_error,
-		       (unsigned)vdata->full_added_to_delete_list,
-		       (unsigned)vdata->full_added_to_vacuum_fetch_list));
+		       (unsigned)vdata->count.db_traverse.total,
+		       (unsigned)vdata->count.db_traverse.skipped,
+		       (unsigned)vdata->count.db_traverse.error,
+		       (unsigned)vdata->count.db_traverse.scheduled));
 	}
 
-	return 0;
+	return;
 }
 
 /**
@@ -799,8 +771,8 @@ static int ctdb_vacuum_db_full(struct ctdb_db_context *ctdb_db,
  * For records for which we are not the lmaster, tell the lmaster to
  * fetch the record.
  */
-static int ctdb_process_vacuum_fetch_lists(struct ctdb_db_context *ctdb_db,
-					   struct vacuum_data *vdata)
+static void ctdb_process_vacuum_fetch_lists(struct ctdb_db_context *ctdb_db,
+					    struct vacuum_data *vdata)
 {
 	int i;
 	struct ctdb_context *ctdb = ctdb_db->ctdb;
@@ -830,11 +802,10 @@ static int ctdb_process_vacuum_fetch_lists(struct ctdb_db_context *ctdb_db,
 			DEBUG(DEBUG_ERR, (__location__ " Failed to send vacuum "
 					  "fetch message to %u\n",
 					  ctdb->nodes[i]->pnn));
-			return -1;
 		}
 	}
 
-	return 0;
+	return;
 }
 
 /**
@@ -860,8 +831,8 @@ static int ctdb_process_vacuum_fetch_lists(struct ctdb_db_context *ctdb_db,
  *  3) The lmaster locally deletes its copies of all records that
  *     could successfully be deleted remotely in step #2.
  */
-static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
-				    struct vacuum_data *vdata)
+static void ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
+				     struct vacuum_data *vdata)
 {
 	int ret, i;
 	struct ctdb_context *ctdb = ctdb_db->ctdb;
@@ -871,18 +842,19 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 	uint32_t *active_nodes;
 	int num_active_nodes;
 	TALLOC_CTX *tmp_ctx;
+	uint32_t sum;
 
-	if (vdata->delete_count == 0) {
-		return 0;
+	if (vdata->count.delete_list.total == 0) {
+		return;
 	}
 
 	tmp_ctx = talloc_new(vdata);
 	if (tmp_ctx == NULL) {
 		DEBUG(DEBUG_ERR,(__location__ " Out of memory\n"));
-		return 0;
+		return;
 	}
 
-	vdata->delete_left = vdata->delete_count;
+	vdata->count.delete_list.left = vdata->count.delete_list.total;
 
 	/*
 	 * get the list of currently active nodes
@@ -894,7 +866,6 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 				   &nodemap);
 	if (ret != 0) {
 		DEBUG(DEBUG_ERR,(__location__ " unable to get node map\n"));
-		ret = -1;
 		goto done;
 	}
 
@@ -921,7 +892,6 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 	recs = talloc_zero(tmp_ctx, struct delete_records_list);
 	if (recs == NULL) {
 		DEBUG(DEBUG_ERR,(__location__ " Out of memory\n"));
-		ret = -1;
 		goto done;
 	}
 	recs->records = (struct ctdb_marshall_buffer *)
@@ -929,7 +899,6 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 				 offsetof(struct ctdb_marshall_buffer, data));
 	if (recs->records == NULL) {
 		DEBUG(DEBUG_ERR,(__location__ " Out of memory\n"));
-		ret = -1;
 		goto done;
 	}
 	recs->records->db_id = ctdb_db->db_id;
@@ -943,8 +912,12 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 	 * records' RSNs in the database, to ensure we (as dmaster)
 	 * keep the highest RSN of the records in the cluster.
 	 */
-	trbt_traversearray32(vdata->delete_list, 1,
-			     delete_marshall_traverse_first, recs);
+	ret = trbt_traversearray32(vdata->delete_list, 1,
+				   delete_marshall_traverse_first, recs);
+	if (ret != 0) {
+		DEBUG(DEBUG_ERR, (__location__ " Error traversing the "
+		      "delete list for first marshalling.\n"));
+	}
 
 	indata.dsize = talloc_get_size(recs->records);
 	indata.dptr  = (void *)recs->records;
@@ -963,7 +936,6 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 			DEBUG(DEBUG_ERR, ("Error storing record copies on "
 					  "node %u: ret[%d] res[%d]\n",
 					  active_nodes[i], ret, res));
-			ret = -1;
 			goto done;
 		}
 
@@ -987,7 +959,6 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 
 			if (recdata.dsize < sizeof(struct ctdb_ltdb_header)) {
 				DEBUG(DEBUG_CRIT,(__location__ " bad ltdb record\n"));
-				ret = -1;
 				goto done;
 			}
 			rechdr = (struct ctdb_ltdb_header *)recdata.dptr;
@@ -1005,15 +976,23 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 				 * update statistics.
 				 */
 				talloc_free(dd);
-				vdata->delete_remote_error++;
-				vdata->delete_left--;
+				vdata->count.delete_list.remote_error++;
+				vdata->count.delete_list.left--;
+			} else {
+				DEBUG(DEBUG_ERR, (__location__ " Failed to "
+				      "find record with hash 0x%08x coming "
+				      "back from RECEIVE_RECORDS "
+				      "control in delete list.\n",
+				      ctdb_hash(&reckey)));
+				vdata->count.delete_list.local_error++;
+				vdata->count.delete_list.left--;
 			}
 
 			rec = (struct ctdb_rec_data *)(rec->length + (uint8_t *)rec);
 		}
 	}
 
-	if (vdata->delete_left == 0) {
+	if (vdata->count.delete_list.left == 0) {
 		goto success;
 	}
 
@@ -1036,13 +1015,16 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 				 offsetof(struct ctdb_marshall_buffer, data));
 	if (recs->records == NULL) {
 		DEBUG(DEBUG_ERR,(__location__ " Out of memory\n"));
-		ret = -1;
 		goto done;
 	}
 	recs->records->db_id = ctdb_db->db_id;
 
-	trbt_traversearray32(vdata->delete_list, 1,
-			     delete_marshall_traverse, recs);
+	ret = trbt_traversearray32(vdata->delete_list, 1,
+				   delete_marshall_traverse, recs);
+	if (ret != 0) {
+		DEBUG(DEBUG_ERR, (__location__ " Error traversing the "
+		      "delete list for second marshalling.\n"));
+	}
 
 	indata.dsize = talloc_get_size(recs->records);
 	indata.dptr  = (void *)recs->records;
@@ -1061,7 +1043,6 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 			DEBUG(DEBUG_ERR, ("Failed to delete records on "
 					  "node %u: ret[%d] res[%d]\n",
 					  active_nodes[i], ret, res));
-			ret = -1;
 			goto done;
 		}
 
@@ -1085,7 +1066,6 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 
 			if (recdata.dsize < sizeof(struct ctdb_ltdb_header)) {
 				DEBUG(DEBUG_CRIT,(__location__ " bad ltdb record\n"));
-				ret = -1;
 				goto done;
 			}
 			rechdr = (struct ctdb_ltdb_header *)recdata.dptr;
@@ -1103,15 +1083,23 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 				 * the tree and update statistics.
 				 */
 				talloc_free(dd);
-				vdata->delete_remote_error++;
-				vdata->delete_left--;
+				vdata->count.delete_list.remote_error++;
+				vdata->count.delete_list.left--;
+			} else {
+				DEBUG(DEBUG_ERR, (__location__ " Failed to "
+				      "find record with hash 0x%08x coming "
+				      "back from TRY_DELETE_RECORDS "
+				      "control in delete list.\n",
+				      ctdb_hash(&reckey)));
+				vdata->count.delete_list.local_error++;
+				vdata->count.delete_list.left--;
 			}
 
 			rec = (struct ctdb_rec_data *)(rec->length + (uint8_t *)rec);
 		}
 	}
 
-	if (vdata->delete_left == 0) {
+	if (vdata->count.delete_list.left == 0) {
 		goto success;
 	}
 
@@ -1123,37 +1111,61 @@ static int ctdb_process_delete_list(struct ctdb_db_context *ctdb_db,
 	 * active remote nodes.
 	 */
 
-	trbt_traversearray32(vdata->delete_list, 1,
-			     delete_record_traverse, vdata);
+	ret = trbt_traversearray32(vdata->delete_list, 1,
+				   delete_record_traverse, vdata);
+	if (ret != 0) {
+		DEBUG(DEBUG_ERR, (__location__ " Error traversing the "
+		      "delete list for deletion.\n"));
+	}
 
 success:
 
-	if (vdata->delete_count > 0) {
+	if (vdata->count.delete_list.left != 0) {
+		DEBUG(DEBUG_ERR, (__location__ " Vaccum db[%s] error: "
+		      "there are %u records left for deletion after "
+		      "processing delete list\n",
+		      ctdb_db->db_name,
+		      (unsigned)vdata->count.delete_list.left));
+	}
+
+	sum = vdata->count.delete_list.deleted
+	    + vdata->count.delete_list.skipped
+	    + vdata->count.delete_list.remote_error
+	    + vdata->count.delete_list.local_error
+	    + vdata->count.delete_list.left;
+
+	if (vdata->count.delete_list.total != sum) {
+		DEBUG(DEBUG_ERR, (__location__ " Inconsistency in vacuum "
+		      "delete list counts for db[%s]: total[%u] != sum[%u]\n",
+		      ctdb_db->db_name,
+		      (unsigned)vdata->count.delete_list.total,
+		      (unsigned)sum));
+	}
+
+	if (vdata->count.delete_list.total > 0) {
 		DEBUG(DEBUG_INFO,
 		      (__location__
 		       " vacuum delete list statistics: "
 		       "db[%s] "
-		       "coll[%u] "
+		       "total[%u] "
+		       "del[%u] "
+		       "skip[%u] "
 		       "rem.err[%u] "
 		       "loc.err[%u] "
-		       "skip[%u] "
-		       "del[%u] "
 		       "left[%u]\n",
 		       ctdb_db->db_name,
-		       (unsigned)vdata->delete_count,
-		       (unsigned)vdata->delete_remote_error,
-		       (unsigned)vdata->delete_local_error,
-		       (unsigned)vdata->delete_skipped,
-		       (unsigned)vdata->delete_deleted,
-		       (unsigned)vdata->delete_left));
+		       (unsigned)vdata->count.delete_list.total,
+		       (unsigned)vdata->count.delete_list.deleted,
+		       (unsigned)vdata->count.delete_list.skipped,
+		       (unsigned)vdata->count.delete_list.remote_error,
+		       (unsigned)vdata->count.delete_list.local_error,
+		       (unsigned)vdata->count.delete_list.left));
 	}
 
-	ret = 0;
-
 done:
 	talloc_free(tmp_ctx);
 
-	return ret;
+	return;
 }
 
 /**
@@ -1165,23 +1177,22 @@ static int ctdb_vacuum_init_vacuum_data(struct ctdb_db_context *ctdb_db,
 	int i;
 	struct ctdb_context *ctdb = ctdb_db->ctdb;
 
-	vdata->fast_added_to_delete_list = 0;
-	vdata->fast_added_to_vacuum_fetch_list = 0;
-	vdata->fast_deleted = 0;
-	vdata->fast_skipped = 0;
-	vdata->fast_error = 0;
-	vdata->fast_total = 0;
-	vdata->full_added_to_delete_list = 0;
-	vdata->full_added_to_vacuum_fetch_list = 0;
-	vdata->full_skipped = 0;
-	vdata->full_error = 0;
-	vdata->full_total = 0;
-	vdata->delete_count = 0;
-	vdata->delete_left = 0;
-	vdata->delete_remote_error = 0;
-	vdata->delete_local_error = 0;
-	vdata->delete_skipped = 0;
-	vdata->delete_deleted = 0;
+	vdata->count.delete_queue.added_to_delete_list = 0;
+	vdata->count.delete_queue.added_to_vacuum_fetch_list = 0;
+	vdata->count.delete_queue.deleted = 0;
+	vdata->count.delete_queue.skipped = 0;
+	vdata->count.delete_queue.error = 0;
+	vdata->count.delete_queue.total = 0;
+	vdata->count.db_traverse.scheduled = 0;
+	vdata->count.db_traverse.skipped = 0;
+	vdata->count.db_traverse.error = 0;
+	vdata->count.db_traverse.total = 0;
+	vdata->count.delete_list.total = 0;
+	vdata->count.delete_list.left = 0;
+	vdata->count.delete_list.remote_error = 0;
+	vdata->count.delete_list.local_error = 0;
+	vdata->count.delete_list.skipped = 0;
+	vdata->count.delete_list.deleted = 0;
 
 	/* the list needs to be of length num_nodes */
 	vdata->vacuum_fetch_list = talloc_zero_array(vdata,
@@ -1268,22 +1279,15 @@ static int ctdb_vacuum_db(struct ctdb_db_context *ctdb_db,
 		return ret;
 	}
 
-	ctdb_vacuum_db_fast(ctdb_db, vdata);
-
-	ret = ctdb_vacuum_db_full(ctdb_db, vdata, full_vacuum_run);
-	if (ret != 0) {
-		return ret;
+	if (full_vacuum_run) {
+		ctdb_vacuum_traverse_db(ctdb_db, vdata);
 	}
 
-	ret = ctdb_process_vacuum_fetch_lists(ctdb_db, vdata);
-	if (ret != 0) {
-		return ret;
-	}
+	ctdb_process_delete_queue(ctdb_db, vdata);
 
-	ret = ctdb_process_delete_list(ctdb_db, vdata);
-	if (ret != 0) {
-		return ret;
-	}
+	ctdb_process_vacuum_fetch_lists(ctdb_db, vdata);
+
+	ctdb_process_delete_list(ctdb_db, vdata);
 
 	/* this ensures we run our event queue */
 	ctdb_ctrl_getpnn(ctdb, TIMELIMIT(), CTDB_CURRENT_NODE);
@@ -1295,9 +1299,10 @@ static int ctdb_vacuum_db(struct ctdb_db_context *ctdb_db,
 /*
  * traverse function for repacking
  */
-static int repack_traverse(struct tdb_context *tdb, TDB_DATA key, TDB_DATA data, void *private)
+static int repack_traverse(struct tdb_context *tdb, TDB_DATA key, TDB_DATA data,
+			   void *private_data)
 {
-	struct vacuum_data *vdata = (struct vacuum_data *)private;
+	struct vacuum_data *vdata = (struct vacuum_data *)private_data;
 
 	if (vdata->vacuum) {
 		uint32_t hash = ctdb_hash(&key);
@@ -1319,7 +1324,7 @@ static int repack_traverse(struct tdb_context *tdb, TDB_DATA key, TDB_DATA data,
 				hdr->dmaster == kd->ctdb->pnn &&
 				ctdb_lmaster(kd->ctdb, &(kd->key)) == kd->ctdb->pnn &&
 				kd->hdr.rsn == hdr->rsn) {
-				vdata->vacuumed++;
+				vdata->count.repack.vacuumed++;
 				return 0;
 			}
 		}
@@ -1328,7 +1333,7 @@ static int repack_traverse(struct tdb_context *tdb, TDB_DATA key, TDB_DATA data,
 		vdata->traverse_error = true;
 		return -1;
 	}
-	vdata->copied++;
+	vdata->count.repack.copied++;
 	return 0;
 }
 
@@ -1356,8 +1361,8 @@ static int ctdb_repack_tdb(struct tdb_context *tdb, TALLOC_CTX *mem_ctx, struct
 	vdata->traverse_error = false;
 	vdata->dest_db = tmp_db;
 	vdata->vacuum = true;
-	vdata->vacuumed = 0;
-	vdata->copied = 0;
+	vdata->count.repack.vacuumed = 0;
+	vdata->count.repack.copied = 0;
 
 	/*
 	 * repack and vacuum on-the-fly by not writing the records that are
@@ -1370,7 +1375,8 @@ static int ctdb_repack_tdb(struct tdb_context *tdb, TALLOC_CTX *mem_ctx, struct
 		return -1;		
 	}
 
-	DEBUG(DEBUG_INFO,(__location__ " %u records vacuumed\n", vdata->vacuumed));
+	DEBUG(DEBUG_INFO,(__location__ " %u records vacuumed\n",
+	      vdata->count.repack.vacuumed));
 	
 	if (vdata->traverse_error) {
 		DEBUG(DEBUG_ERR,(__location__ " Error during traversal\n"));
@@ -1389,7 +1395,7 @@ static int ctdb_repack_tdb(struct tdb_context *tdb, TALLOC_CTX *mem_ctx, struct
 	vdata->traverse_error = false;
 	vdata->dest_db = tdb;
 	vdata->vacuum = false;
-	vdata->copied = 0;
+	vdata->count.repack.copied = 0;
 
 	if (tdb_traverse_read(tmp_db, repack_traverse, vdata) == -1) {
 		DEBUG(DEBUG_ERR,(__location__ " Failed to traverse copying back\n"));
@@ -1412,7 +1418,8 @@ static int ctdb_repack_tdb(struct tdb_context *tdb, TALLOC_CTX *mem_ctx, struct
 		DEBUG(DEBUG_ERR,(__location__ " Failed to commit\n"));
 		return -1;
 	}
-	DEBUG(DEBUG_INFO,(__location__ " %u records copied\n", vdata->copied));
+	DEBUG(DEBUG_INFO,(__location__ " %u records copied\n",
+	      vdata->count.repack.copied));
 
 	return 0;
 }
@@ -1426,17 +1433,10 @@ static int ctdb_vacuum_and_repack_db(struct ctdb_db_context *ctdb_db,
 				     bool full_vacuum_run)
 {
 	uint32_t repack_limit = ctdb_db->ctdb->tunable.repack_limit;
-	uint32_t vacuum_limit = ctdb_db->ctdb->tunable.vacuum_limit;
 	const char *name = ctdb_db->db_name;
-	int freelist_size;
+	int freelist_size = 0;
 	struct vacuum_data *vdata;
 
-	freelist_size = tdb_freelist_size(ctdb_db->ltdb->tdb);
-	if (freelist_size == -1) {
-		DEBUG(DEBUG_ERR,(__location__ " Failed to get freelist size for '%s'\n", name));
-		return -1;
-	}
-
 	vdata = talloc_zero(mem_ctx, struct vacuum_data);
 	if (vdata == NULL) {
 		DEBUG(DEBUG_ERR,(__location__ " Out of memory\n"));
@@ -1444,7 +1444,6 @@ static int ctdb_vacuum_and_repack_db(struct ctdb_db_context *ctdb_db,
 	}
 
 	vdata->ctdb = ctdb_db->ctdb;
-	vdata->vacuum_limit = vacuum_limit;
 	vdata->repack_limit = repack_limit;
 	vdata->delete_list = trbt_create(vdata, 0);
 	vdata->ctdb_db = ctdb_db;
@@ -1463,17 +1462,26 @@ static int ctdb_vacuum_and_repack_db(struct ctdb_db_context *ctdb_db,
 		DEBUG(DEBUG_ERR,(__location__ " Failed to vacuum '%s'\n", name));
 	}
 
+	if (repack_limit != 0) {
+		freelist_size = tdb_freelist_size(ctdb_db->ltdb->tdb);
+		if (freelist_size == -1) {
+			DEBUG(DEBUG_ERR,(__location__ " Failed to get freelist size for '%s'\n", name));
+			talloc_free(vdata);
+			return -1;
+		}
+	}
+
 	/*
 	 * decide if a repack is necessary
 	 */
-	if (freelist_size < repack_limit && vdata->delete_left < vacuum_limit)
+	if ((repack_limit == 0 || (uint32_t)freelist_size < repack_limit))
 	{
 		talloc_free(vdata);
 		return 0;
 	}
 
 	DEBUG(DEBUG_INFO,("Repacking %s with %u freelist entries and %u records to delete\n", 
-			name, freelist_size, vdata->delete_left));
+			name, freelist_size, vdata->count.delete_list.left));
 
 	/*
 	 * repack and implicitely get rid of the records we can delete
