@@ -456,6 +456,10 @@ static void ctdb_run_debug_hung_script(struct ctdb_context *ctdb, struct debug_h
 		return;
 	}
 
+	if (getenv("CTDB_DEBUG_HUNG_SCRIPT") != NULL) {
+		debug_hung_script = getenv("CTDB_DEBUG_HUNG_SCRIPT");
+	}
+
 	argv = talloc_array(state, const char *, 5);
 
 	argv[0] = talloc_asprintf(argv, "%d", fd[1]);
@@ -897,8 +901,6 @@ static void run_eventscripts_callback(struct ctdb_context *ctdb, int status,
 	struct eventscript_callback_state *state = 
 		talloc_get_type(private_data, struct eventscript_callback_state);
 
-	ctdb_enable_monitoring(ctdb);
-
 	if (status != 0) {
 		DEBUG(DEBUG_ERR,(__location__ " Failed to run eventscripts\n"));
 	}
@@ -962,14 +964,11 @@ int32_t ctdb_run_eventscripts(struct ctdb_context *ctdb,
 
 	DEBUG(DEBUG_NOTICE,("Running eventscripts with arguments %s\n", indata.dptr));
 
-	ctdb_disable_monitoring(ctdb);
-
 	ret = ctdb_event_script_callback(ctdb, 
 			 state, run_eventscripts_callback, state,
 			 call, "%s", options);
 
 	if (ret != 0) {
-		ctdb_enable_monitoring(ctdb);
 		DEBUG(DEBUG_ERR,(__location__ " Failed to run eventscripts with arguments %s\n", indata.dptr));
 		talloc_free(state);
 		return -1;
